package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.beans.PropertyVetoException;
import java.util.HashMap;
import java.util.Map;

import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Configurable;
import de.ksquared.jds.components.Configurable.Option.OptionType;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactList;
import de.ksquared.jds.contacts.ContactList.Alignment;
import de.ksquared.jds.contacts.ContactUtilities;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;


public class Demultiplexer extends Component implements Sociable, Configurable {
	
	private static final long serialVersionUID = -4306382459038041642L;

	public static final ComponentAttributes componentAttributes = 
			new ComponentAttributes(
					"Demultiplexer",
					"group.selector",
					"Demultiplexer",
					"Malte Mues",
					1);
	
	private int controlSignals = 2;
	
	private final int rectHeight = 100;
	private final int rectWidth = 60;
	private final int polyHeight = 20;
	
	public Demultiplexer() {
		setNewContacts();
	}

	@Override
	public void paint(Graphics graphics) {
		
		Dimension size = getSize();
		graphics.setColor(Color.BLACK);
		graphics.drawRect(5, 0, size.width-10, size.height-polyHeight-5);
		
		int[] XArray = {5, size.width-5, size.width-5-8, 5+8};
	   int[] YArray = {size.height-polyHeight-5, size.height-polyHeight-5, size.height-5, size.height-5};
	    
		graphics.drawPolygon(XArray, YArray, 4);

		graphics.drawString("Demux", 10, 35);
		
	}

	@Override
	public Dimension getSize() {
		
		int width = rectWidth;
		int height = rectHeight+polyHeight;
		
		if(controlSignals >= 4)	
		{
			width = 60+((controlSignals-4)*3);
			height = polyHeight+10+(int)(Math.pow(2, controlSignals)*8);
		}
	
		return new Dimension(width, height);
	}

	
	private InputContact input;
	private ContactList<InputContact> controls;
	private ContactList<OutputContact> outputs;

	@Override
	public Contact[] getContacts() {
		return ContactUtilities.concatenateContacts(
				ContactUtilities.concatenateContacts(input, outputs.toArray()),
				controls.toArray());
	}

	
	@Override
	public void calculate() {
		int a = 0;
		int b= controlSignals-1;
		for (InputContact x: controls) {
			if (x.isCharged()) {
				a = a + (int) Math.pow(2, b);
			}
			b--;
		}
		for (OutputContact x : outputs) {
			x.setCharged(false);
		}
		outputs.getContact(a).setCharged(input.isCharged());
	}

	@Override
	public Option[] getOptions() {
		return new Option[] {
		new Option("numbers", "Kontrollsignale", OptionType.NUMBER, 2)

		};
	}

	@Override
	public Map<Option, Object> getConfiguration() {
		Option[] options = getOptions();
		Map<Option, Object> configuration = new HashMap<Option, Object>();
		configuration.put(options[0], this.controlSignals);
		return configuration;
	}

	@Override
	public void setConfiguration(Map<Option, Object> configuration)
			throws PropertyVetoException {
		Option[] options = getOptions();
		int option0 = (Integer) configuration.get(options[0]);
		if (option0 < 1 || option0 > 7) {
			throw new PropertyVetoException(
					"Es sind nur zwschischen 1 und 7 Kontrollsignale erlaubt, einschließlich 1 und 7", null);
		}
		this.controlSignals = (Integer) configuration.get(options[0]);
		setNewContacts();

	}
	private void setNewContacts(){
		Dimension dimension = getSize();
			outputs = new ContactList<OutputContact>(this, OutputContact.class);
			outputs.setContacts((int) Math.pow(2, controlSignals));
			outputs.setContactLocations(new Point((int) dimension.getWidth(), 0), Alignment.VERTICAL);

			controls = new ContactList<InputContact>(this, InputContact.class, controlSignals);
			controls.setContactLocations(new Point(0, (int)dimension.getHeight()),
					Alignment.HORIZONTAL);

			input = new InputContact(this, new Point(0, (int)(dimension.getHeight()-5)/2));
	}
}

