package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;

import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Interactable;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactList;
import de.ksquared.jds.contacts.ContactList.Alignment;
import de.ksquared.jds.contacts.OutputContact;



public class HexInput extends Component implements Sociable,Interactable {
	
	private static final long serialVersionUID = 5522677093104410914L;
	
	public static final ComponentAttributes componentAttributes = 
			new ComponentAttributes(
					"Hexinput", 
					"group.switches", 
					"Hexinput",
					"Malte Mues", 2);
	
	private final int width = 80; //should be dividable by 4
	private final int height = 88;
	
	
	
	private ContactList<OutputContact> outputs;
	private transient int currentSelection = 0;
	
	public HexInput () {
		outputs = new ContactList<OutputContact>(this, OutputContact.class, 4);
		outputs.setContactLocations(new Point(getSize().width,0), Alignment.VERTICAL);
	}
	
	
	@Override
	public Contact[] getContacts() {
		return outputs.toArray();
	}
	
	
	@Override
	public void paint(Graphics graphics) {
		Font previousFont = graphics.getFont();
		graphics.setFont(new Font(previousFont.getName(), previousFont.getStyle(), 15));
		graphics.setColor(Color.BLACK);
		graphics.drawRect(0,0,width,height);
		graphics.drawLine(0, height/4*1, width, height/4*1);
		graphics.drawLine(0, height/4*2, width, height/4*2);
		graphics.drawLine(0, height/4*3, width, height/4*3);
		graphics.drawLine(width/4*1, 0, width/4*1, height);
		graphics.drawLine(width/4*2, 0, width/4*2, height);
		graphics.drawLine(width/4*3, 0, width/4*3, height);
		graphics.drawString("0",getPositionX(0),getPositionY(0));
		graphics.drawString("1",getPositionX(1),getPositionY(1));
		graphics.drawString("2",getPositionX(2),getPositionY(2));
		graphics.drawString("3",getPositionX(3),getPositionY(3));
		graphics.drawString("4",getPositionX(4),getPositionY(4));
		graphics.drawString("5",getPositionX(5),getPositionY(5));
		graphics.drawString("6",getPositionX(6),getPositionY(6));
		graphics.drawString("7",getPositionX(7),getPositionY(7));
		graphics.drawString("8",getPositionX(8),getPositionY(8));
		graphics.drawString("9",getPositionX(9),getPositionY(9));
		graphics.drawString("A",getPositionX(10),getPositionY(10));
		graphics.drawString("B",getPositionX(11),getPositionY(11));
		graphics.drawString("C",getPositionX(12),getPositionY(12));
		graphics.drawString("D",getPositionX(13),getPositionY(13));
		graphics.drawString("E",getPositionX(14),getPositionY(14));
		graphics.drawString("F",getPositionX(15),getPositionY(15));
		
		//highlight selected value
		graphics.setColor(Color.RED);
		graphics.drawRect(width/4 * (currentSelection%4), height/4 * (currentSelection/4), width/4, height/4);
		
		
		graphics.setFont(previousFont);
	}
	
	private int getPositionX(int number) {
		int column = number%4;
		return width/4*column + width/8 - 4;
	}
	
	private int getPositionY(int number) {
		int row = number/4;
		return height/4*row + height/8 + 6;
	}
		

	@Override 
	public Dimension getSize() { 
		return new Dimension(width + 5, height);
	}

	
	@Override 
	public void calculate() {
	}
	
	
	@Override
	public void mouseClick(MouseEvent event){
		Point pointMouse = event.getPoint();
		int x = (int) pointMouse.getX();
		int y = (int) pointMouse.getY();
		currentSelection = x / (width/4) + 4 * (y / (height/4));
		int i = 0;        
		for (OutputContact c: outputs) {
            c.setCharged(((1 << i) & currentSelection) != 0);
            i++;       
		}
	}
	
	
	@Override
	public void mouseDoubleClick(MouseEvent event) {
	}
	
	@Override
	public void mouseDown(MouseEvent event) {
	}
	
	@Override
	public void mouseUp(MouseEvent event) {
	}
}
