package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.beans.PropertyVetoException;
import java.util.HashMap;
import java.util.Map;

import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Configurable;
import de.ksquared.jds.components.Configurable.Option.OptionType;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactList;
import de.ksquared.jds.contacts.ContactList.Alignment;
import de.ksquared.jds.contacts.ContactUtilities;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;

public class Paralleladdierer extends Component implements Sociable, Configurable
{
	
	private static final long serialVersionUID = 3384040919726416638L;

	public static final ComponentAttributes componentAttributes = 
		new ComponentAttributes("Paralleladdierer",
										"group.alu",
										"Paralleladdierer",
										"Johannes Linke",
										1);
	
	private int numInputBits=4;
	
	private ContactList<InputContact> inputs;
	private ContactList<OutputContact> outputs;
	
	
	public Paralleladdierer()
	{
		setNewContacts();
	}
	
	
	@Override
	public Dimension getSize()
	{
		return new Dimension(80,numInputBits * 2 * 18); 
	}
	
	
	@Override
	public void paint(Graphics graphics)
	{
		graphics.setColor(Color.BLACK);
		Font previousFont = graphics.getFont();
		graphics.drawString("VA", getSize().width/2 - 8, getSize().height/2);
		graphics.drawString(numInputBits + "-Bit", getSize().width/2 - 12, getSize().height/2 + 20);
		graphics.setFont(new Font(previousFont.getName(), previousFont.getStyle(), 10));
		
		int fontHeight = graphics.getFontMetrics().getHeight() - 8;
		
		int yPosInputCarry = (int) inputs.getContact(inputs.getContactsCount()-1).getLocation().getY();
		graphics.drawString("Ce", 9, yPosInputCarry + fontHeight/2);
		
		for (int i = 0; i < inputs.getContactsCount() - 1; i++) {
			int yPos = inputs.getContact(i).getLocation().y;
			String text = ((i < numInputBits) ? "A" : "B") + i % numInputBits;
			graphics.drawString(text, 9, yPos + fontHeight/2);
		}
		
		int yPosOutputCarry = (int) outputs.getContact(outputs.getContactsCount()-1).getLocation().getY();
		graphics.drawString("C", getSize().width-17, yPosOutputCarry + fontHeight/2);
		
		for (int i = 0; i < outputs.getContactsCount() - 1; i++) {
			int yPos = outputs.getContact(i).getLocation().y;
			int xPos = getSize().width;
			String text = "S" + i;
			graphics.drawString(text, xPos - 23, yPos + fontHeight/2);
		}
		graphics.drawRect(5,0,getSize().width-10,getSize().height);
		
		graphics.setFont(previousFont);
	}
	
	
	@Override
	public Contact[] getContacts()
	{
		return ContactUtilities.concatenateContacts(outputs.toArray(), inputs.toArray());
	}

	@Override
	public void calculate()
	{
		int result = 0;
		if (inputs.getContact(inputs.getContactsCount()-1).isCharged())
			result++;
		
		for (int i = 0; i < inputs.getContactsCount() - 1; i++) {
			if (inputs.getContact(i).isCharged()) {
				result += ((1 % numInputBits) << i % numInputBits );
			}
		}
		
		for (int i = 0; i < outputs.getContactsCount()-1; i++) {
			outputs.getContact(i).setCharged((result & (1 << i)) != 0);
		}
		
		outputs.getContact(outputs.getContactsCount()-1).setCharged(result >= 1 << numInputBits);
	}
	
	
	@Override
	public Map<Option, Object> getConfiguration()
	{
		Option[] options = getOptions();
		Map<Option,Object> configuration = new HashMap<Option,Object>(); 
		configuration.put(options[0],numInputBits); 
		return configuration;
	}
	
	
	@Override
	public Option[] getOptions()
	{
		return new Option[]{
				new Option("number","Anzahl der Eingänge",OptionType.NUMBER,4)
				};
	}
	
	
	@Override
	public void setConfiguration(Map<Option, Object> configuration)
			throws PropertyVetoException
	{
		Option[] options = getOptions(); 
		int option0 = (Integer) configuration.get(options[0]);
		if (option0 < 2 || option0 > 8) {
			throw new PropertyVetoException(
				"Bitte eine Zahl zwischen 2 und 8 eingeben, einschließlich 2 und 8.", null);
		}
		this.numInputBits = option0;
		setNewContacts();
	}
	
	
	private void setNewContacts()
	{
			Dimension size = getSize();
		
			inputs=new ContactList<InputContact>(this, InputContact.class);
			inputs.setContacts(numInputBits*2 + 1);
			inputs.setContactLocations(Alignment.VERTICAL);
		
			outputs=new ContactList<OutputContact>(this, OutputContact.class);
			outputs.setContacts(numInputBits + 1);
			outputs.setContactLocations(new Point(size.width, 0), Alignment.VERTICAL);
	}
}
